@@ -74,6 +74,9 @@ copy_xattr (const char *from, const char *to)
       if (size < 0)
 	{
 	  fprintf (stderr, _("listing attributes of %s\n"), from);
+	  if (value)
+	    free (value);
+	  free (names);
 	  return -1;
 	}
       else
@@ -96,14 +99,18 @@ copy_xattr (const char *from, const char *to)
 	    {
 	      fprintf (stderr, _("getting attribute %s of %s\n"),
 		       name, from);
+	      if (value)
+		free (value);
+	      free (names);
 	      return -1;
 	    }
 
 	  value = realloc (old_value = value, size);
 	  if (size != 0 && value == NULL)
 	    {
-	      free (old_value);
 	      fputs ("running out of memory!\n", stderr);
+	      free (old_value);
+	      free (names);
 	      return -1;
 	    }
 
@@ -112,25 +119,27 @@ copy_xattr (const char *from, const char *to)
 	    {
 	      fprintf (stderr, _("getting attribute %s of %s\n"),
 		       name, from);
+	      if (value)
+		free (value);
+	      free (names);
 	      return -1;
 	    }
 
 	  if (lsetxattr (to, name, value, size, 0) != 0)
 	    {
 	      if (errno == ENOSYS)
-		{
-		  fprintf (stderr, _("setting attributes for %s\n"), to);
-		  return -1;
-		}
+		fprintf (stderr, _("setting attributes for %s\n"), to);
 	      else
-		{
-		  fprintf (stderr, _("setting attribute %s for %s\n"),
-			   name, to);
-		  return -1;
-		}
+		fprintf (stderr, _("setting attribute %s for %s\n"),
+			 name, to);
+	      if (value)
+		free (value);
+	      free (names);
+	      return -1;
 	    }
 	}
-      free (value);
+      if (value)
+	free (value);
       free (names);
     }
 #elif defined(WITH_SELINUX)
