@@ -374,7 +374,7 @@ write_group_data (group_t *data, int is_locked)
 	    }
           fchmod (newgf_fd, group_stat.st_mode);
           fchown (newgf_fd, group_stat.st_uid, group_stat.st_gid);
-	  if (copy_xattr (group_orig, group_tmp) > 0)
+	  if (copy_xattr (group_orig, group_tmp) != 0)
 	    {
 	      fclose (oldgf);
 	      close (newgf_fd);
@@ -569,7 +569,7 @@ write_group_data (group_t *data, int is_locked)
             }
           fchmod (newgf_fd, group_stat.st_mode);
           fchown (newgf_fd, group_stat.st_uid, group_stat.st_gid);
-	  if (copy_xattr (gshadow_orig, gshadow_tmp) > 0)
+	  if (copy_xattr (gshadow_orig, gshadow_tmp) != 0)
 	    {
 	      fclose (oldgf);
 	      close (newgf_fd);
