@@ -20,10 +20,12 @@
 
 #define _GNU_SOURCE
 
+#include <errno.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 
 #include "logindefs.h"
 
@@ -71,16 +73,16 @@ search (const char *name)
 }
 
 /* Load the login.defs file (/etc/login.defs)  */
-static int
-load_defaults (void)
+static void
+load_defaults_internal (const char *filename)
 {
   FILE *fp;
   char *buf = NULL;
   size_t buflen = 0;
 
-  fp = fopen ("/etc/login.defs", "r");
+  fp = fopen (filename, "r");
   if (NULL == fp)
-    return 1;
+    return;
 
   while (!feof (fp))
     {
@@ -120,9 +122,9 @@ load_defaults (void)
       if (cp[strlen (cp) - 1] == '\n')
         cp[strlen (cp) - 1] = '\0';
 
-      tmp = strsep (&cp, " \t");
+      tmp = strsep (&cp, " \t=");
       if (cp != NULL)
-	while (isspace ((int)*cp))
+	while (isspace ((int)*cp) || *cp == '=')
 	  ++cp;
 
       store (tmp, cp);
@@ -131,8 +133,13 @@ load_defaults (void)
 
   if (buf)
     free (buf);
+}
 
-  return 0;
+static void
+load_defaults (void)
+{
+  load_defaults_internal ("/etc/default/passwd");
+  load_defaults_internal ("/etc/login.defs");
 }
 
 int
@@ -167,7 +174,8 @@ getlogindefs_num (const char *name, long dflt)
     return dflt;
 
   retval = strtol (val, &cp, 0);
-  if (*cp != '\0')
+  if (*cp != '\0' ||
+      ((retval == LONG_MAX || retval == LONG_MIN) && errno == ERANGE))
     {
       fprintf (stderr,
 	       "login.defs: %s contains invalid numerical value: %s!\n",
@@ -182,7 +190,7 @@ getlogindefs_unum (const char *name, unsigned long dflt)
 {
   const char *val;
   char *cp;
-  long retval;
+  unsigned long retval;
 
   if (list == NULL)
     load_defaults ();
@@ -193,7 +201,7 @@ getlogindefs_unum (const char *name, unsigned long dflt)
     return dflt;
 
   retval = strtoul (val, &cp, 0);
-  if (*cp != '\0')
+  if (*cp != '\0' || (retval == ULONG_MAX && errno == ERANGE))
     {
       fprintf (stderr,
 	       "login.defs: %s contains invalid numerical value: %s!\n",
@@ -221,6 +229,7 @@ getlogindefs_str (const char *name, const char *dflt)
 int
 main ()
 {
+  printf ("CYPT=%s\n", getlogindefs_str ("cRypt", "no"));
   printf ("LOG_UNKFAIL_ENAB=%s\n", getlogindefs_str ("log_unkfail_enab",""));
   printf ("DOESNOTEXIST=%s\n", getlogindefs_str ("DOESNOTEXIST","yes"));
   return 0;
