@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 Thorsten Kukuk
+/* Copyright (C) 2003, 2004 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -32,6 +32,7 @@
 #include <libintl.h>
 
 #include "public.h"
+#include "utf8conv.h"
 #include "error_codes.h"
 
 #ifndef _
@@ -103,7 +104,8 @@ hp_expire (const struct passwd *pw)
   if (max < 0)
     {
     error_state:
-      fprintf (stderr, _("Age field for %s is wrong"), pw->pw_name);
+      fprintf (stderr, _("Age field for %s is wrong"),
+	       utf8_to_locale (pw->pw_name));
       return -1;
     }
   ++age;
@@ -231,7 +233,7 @@ main (int argc, char *argv[])
           print_help (program);
           return 0;
         case 'v':
-          print_version (program, "2003");
+          print_version (program, "2004");
           return 0;
         case 'u':
           print_usage (stdout, program);
