@@ -1,4 +1,4 @@
-/* Copyright (C) 2002, 2003 Thorsten Kukuk
+/* Copyright (C) 2002, 2003, 2004 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -50,6 +50,7 @@
 #include "logindefs.h"
 #include "read-files.h"
 #include "error_codes.h"
+#include "parse_crypt_arg.h"
 
 #define MAX_PASSWD_TRIES 3
 
@@ -77,106 +78,13 @@ print_help (const char *program)
   fputs (_("Valid services for -r are: files, nis, nisplus, ldap\n"), stdout);
 }
 
-/* XXX move in library. */
-#ifndef RANDOM_DEVICE
-#define RANDOM_DEVICE "/dev/urandom"
-#endif
-
-#if defined(HAVE_CRYPT_GENSALT_RN)
-static int
-read_loop (int fd, char *buffer, int count)
-{
-  int offset, block;
-
-  offset = 0;
-  while (count > 0)
-    {
-      block = read(fd, &buffer[offset], count);
-
-      if (block < 0)
-        {
-          if (errno == EINTR)
-            continue;
-          return block;
-        }
-      if (!block)
-        return offset;
-
-      offset += block;
-      count -= block;
-    }
-
-  return offset;
-}
-#endif
-
-static char *
-#if defined(HAVE_CRYPT_GENSALT_RN)
-make_crypt_salt (const char *crypt_prefix, int crypt_rounds)
-#else
-make_crypt_salt (const char *crypt_prefix __attribute__ ((unused)),
-                 int crypt_rounds __attribute__ ((unused)))
-#endif
-{
-#if defined(HAVE_CRYPT_GENSALT_RN)
-#define CRYPT_GENSALT_OUTPUT_SIZE (7 + 22 + 1)
-  int fd;
-  char entropy[16];
-  char *retval;
-  char output[CRYPT_GENSALT_OUTPUT_SIZE];
-
-  fd = open (RANDOM_DEVICE, O_RDONLY);
-  if (fd < 0)
-    {
-      fprintf (stderr, _("Can't open %s for reading: %s\n"),
-               RANDOM_DEVICE, strerror (errno));
-      return NULL;
-    }
-
-  if (read_loop (fd, entropy, sizeof(entropy)) != sizeof(entropy))
-    {
-      close (fd);
-      fprintf (stderr, _("Unable to obtain entropy from %s\n"),
-               RANDOM_DEVICE);
-      return NULL;
-    }
-
-  close (fd);
-
-  retval = crypt_gensalt_rn (crypt_prefix, crypt_rounds, entropy,
-                             sizeof (entropy), output, sizeof(output));
-
-  memset (entropy, 0, sizeof (entropy));
-
-  if (!retval)
-    {
-      fprintf (stderr,
-               _("Unable to generate a salt, check your crypt settings.\n"));
-      return NULL;
-    }
-
-  return strdup (retval);
-#else
-#define ascii_to_bin(c) ((c)>='a'?(c-59):(c)>='A'?((c)-53):(c)-'.')
-#define bin_to_ascii(c) ((c)>=38?((c)-38+'a'):(c)>=12?((c)-12+'A'):(c)+'.')
-
-  time_t tm;
-  char salt[3];
-
-  time (&tm);
-  salt[0] = bin_to_ascii(tm & 0x3f);
-  salt[1] = bin_to_ascii((tm >> 6) & 0x3f);
-  salt[2] = '\0';
-
-  return strdup (salt);
-#endif
-}
-
 int
 main (int argc, char **argv)
 {
   const char *program = "gpasswd";
   char *group;
+  const char *crypt_str;
+  crypt_t use_crypt;
   int remove_password = 0;
   char *binddn = NULL;
   int P_flag = 0;
@@ -194,6 +102,11 @@ main (int argc, char **argv)
      signals.  */
   init_environment ();
 
+  crypt_str = getlogindefs_str ("GROUP_CRYPT", NULL);
+  if (crypt_str == NULL)
+    crypt_str = getlogindefs_str ("CRYPT", "des");
+  use_crypt = parse_crypt_arg (crypt_str);
+
   /* Parse program arguments */
   while (1)
     {
@@ -256,7 +169,7 @@ main (int argc, char **argv)
           print_help (program);
           return 0;
         case '\255':
-          print_version (program, "2003");
+          print_version (program, "2004");
           return 0;
         case '\254':
           print_usage (stdout, program);
@@ -377,20 +290,62 @@ main (int argc, char **argv)
 	  char *salt;
           struct crypt_data output;
           memset (&output, 0, sizeof (output));
-	  /* If we don't support passwords longer 8 characters,
-	     truncate them */
-	  if (strlen (p1) > 8)
-	    p1[8] = '\0';
-	  salt =  make_crypt_salt ("", 0);
-	  if (salt != NULL)
-	    gr_data->newpassword = strdup (crypt_r (p1, salt, &output));
-	  else
-	    {
-	      fprintf (stderr, _("Cannot create salt for standard crypt.\n"));
-	      goto abort_change;
-	    }
-	  free (salt);
-	}
+
+
+          switch (use_crypt)
+            {
+            case DES:
+              /* If we don't support passwords longer 8 characters,
+                 truncate them */
+              if (strlen (p1) > 8)
+                p1[8] = '\0';
+              salt =  make_crypt_salt ("", 0);
+              if (salt != NULL)
+                gr_data->newpassword = strdup (crypt_r (p1, salt, &output));
+              else
+                {
+                  fprintf (stderr, _("Cannot create salt for standard crypt"));
+		  goto abort_change;
+                }
+              free (salt);
+              break;
+            case BLOWFISH:
+#if defined(HAVE_CRYPT_GENSALT_RN)
+	      /* blowfish has a limit of 72 characters */
+	      if (use_crypt == BLOWFISH && strlen (p1) > 72)
+		p1[72] = '\0';
+              salt = make_crypt_salt ("$2a$", 0 /* XXX crypt_rounds */);
+              if (salt != NULL)
+                gr_data->newpassword = strdup (crypt_r (p1, salt, &output));
+              else
+                {
+                  fprintf (stderr, _("Cannot create salt for blowfish crypt"));
+		  goto abort_change;
+                }
+              free (salt);
+              break;
+#else
+	      fprintf (stderr,
+		       _("No support for blowfish compiled in. Using MD5\n"));
+#endif
+            case MD5:
+              /* MD5 has a limit of 127 characters */
+              if (strlen (p1) > 127)
+                p1[127] = '\0';
+              salt = make_crypt_salt ("$1$", 0);
+              if (salt != NULL)
+                gr_data->newpassword = strdup (crypt_r (p1, salt, &output));
+              else
+                {
+                  fprintf (stderr, _("Cannot create salt for MD5 crypt"));
+		  goto abort_change;
+                }
+              free (salt);
+              break;
+            default:
+              abort();
+            }
+        }
     }
 
   gr_data->todo = DO_MODIFY;
